/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>

#pragma implementation "hem.hxx"
#pragma implementation "hem.cc"
#pragma implementation "set.hxx"

#include "hem.hxx"
#include "hem.cc"
#include "set.hxx"

#include <sys/types.h>
#include <sys/time.h>
#include <sys/timeb.h>

struct timeval Tbegin;
struct timeval Tend;
struct timeval tt ;

void timeBegin()
{
  gettimeofday(&Tbegin, NULL);
}

void timeEnd()
{
  gettimeofday(&Tend, NULL);
}

double timeDiff()
{
  double res = (Tend.tv_sec - Tbegin.tv_sec) ;
  res += ((float)(Tend.tv_usec)/1000000) - ((float)(Tbegin.tv_usec)/1000000) ;
  return res ;
}

class Page {  
  public :
  int i ;

  Page(int val) : i(val) {} ;
} ;

int main(int argc, char** argv) {

  SetPtr<Page> set ;
  Page* page ;
  int cpt = atoi(argv[1]) ;
  Page* tab[cpt] ;

  IterSetPtr<Page> iter(set) ;
  for (; iter.more(); iter.next()) {
    cout << "Erreur dans l'iteration" << endl ;
  }
  
  timeBegin() ;
  for (int i = 0; i < cpt; i = i + 2) {
    tab[i] = new Page(i) ;
    set += tab[i] ;
  }
  for (i = 1; i < cpt; i = i + 2) {
    tab[i] = new Page(i) ;
    set += tab[i] ;
  }
  timeEnd() ;
  printf ("Temps insertion : %f\n", timeDiff()) ;

  int count = 0 ;
  int checkcount = 0 ;
  iter.reset() ;
  timeBegin() ;
  for (i = 0; iter.more(); iter.next(), i++) {
    checkcount += i ;
    count += iter.get()->i ;
  }
  timeEnd() ;
  printf ("Temps iteration : %f\n", timeDiff()) ;
  if ((i != cpt) || (count != checkcount)) {
    cout << "Erreur dans l'iteration" << endl ;
  }
  
  timeBegin() ;
  for (i = 0; i < cpt; i++) {    
    page = set[tab[i]] ;
    if ( (page != tab[i]) || (page->i != i) ) {
      cout << "Erreur dans l'insertion" << endl ;
    }
  }
  timeEnd() ;
  printf ("Temps recherche : %f\n", timeDiff()) ;
  
  timeBegin() ;
  for (i = 0; i < cpt; i++) {    
    if (set.member(tab[i]) == 0) {
      cout << "Erreur dans la recherche de membre" << endl ;
    }
  }
  timeEnd() ;
  printf ("Temps membre? : %f\n", timeDiff()) ;

  timeBegin() ;
  for (i = cpt; i >= 0; i--) {    
    set -= tab[i] ;
    for (int j = 0; j < cpt; j++) {    
      page = set[tab[j]] ;
      if ( (j >= i) && (page != NULL) ) {
	cout << "Erreur dans l'index" << endl ;
      }
      if ( (j < i) && (page == NULL) ) {
	cout << "Erreur dans l'index" << endl ;
      }
    }
  }
  timeEnd() ;
  printf ("Temps suppression : %f\n", timeDiff()) ;

  timeBegin() ;
  for (i = 0; i < cpt; i++) {    
    if (set.member(tab[i])) {
      cout << "Erreur dans la recherche de membre" << endl ;
    }
  }
  timeEnd() ;

  iter.reset() ;
  for (; iter.more(); iter.next()) {
    cout << "Erreur dans l'iteration" << endl ;
  }

  printf ("Temps membre? : %f\n", timeDiff()) ;
}
  

  
